import 'dart:io';

import 'package:args/command_runner.dart';

import '../foundation/cli_message_printer.dart';
import '../foundation/process_runner.dart';

class CreateCommand extends Command<void> {
  @override
  String get name => 'create';

  @override
  String get description =>
      'Sets up a new Dawn application in the current directory.';

  @override
  String get invocation => 'dawn create <app_name>';

  String get _appName => argResults!.rest.first;

  @override
  void run() {
    if (argResults!.rest.isEmpty) {
      usageException('Specify your application\'s name.\n');
    }

    final directory = Directory('./$_appName');

    if (directory.existsSync()) {
      usageException('Directory $_appName already exists.\n');
    }

    Directory.current = directory..createSync();

    _createFiles();
    _installDependencies();

    printCliMessage('Enjoy Coding!', type: CliMessageType.success);

    printCliMessage(
      'Run the following commands:\n'
      '  cd $_appName\n'
      '  webdev serve',
    );
  }

  void _createFiles() {
    printCliMessage('Creating Files...');

    _createFile(path: './.gitignore', body: _gitIgnore);
    _createFile(path: './README.md', body: _readmeDotMd);
    _createFile(path: './pubspec.yaml', body: _pubspecDotYaml);
    _createFile(path: './analysis_options.yaml', body: _analysisOptionsDotYaml);
    _createFile(path: './web/index.html', body: _indexDotHtml);
    _createFile(path: './web/main.dart', body: _mainDotDart);
    _createFile(path: './web/assets/logo.svg', body: _logoDotSvg);
    _createFile(path: './web/assets/icon.svg', body: _iconDotSvg);
    _createFile(path: './web/assets/title.svg', body: _titleDotSvg);
    _createFile(path: './web/assets/banner.svg', body: _bannerDotSvg);
  }

  void _createFile({
    required final String path,
    required final String body,
  }) {
    File(path)
      ..createSync(recursive: true)
      ..writeAsStringSync(body);

    printCliMessage(
      'Created $path.',
      listItem: true,
      type: CliMessageType.success,
    );
  }

  void _installDependencies() {
    printCliMessage('Installing dependencies...');

    _installDependency('dawn');
    _installDependency('dawn_lints', dev: true);
    _installDependency('build_runner', dev: true);
    _installDependency('build_web_compilers', dev: true);
  }

  void _installDependency(final String name, {final bool dev = false}) {
    runProcess(
      'dart',
      ['pub', 'add', if (dev) '-d', name],
      throwOnError: false,
      onSuccess: () => printCliMessage(
        'Installed $name.',
        listItem: true,
        type: CliMessageType.success,
      ),
      onError: () => printCliMessage(
        'Couldn\'t install $name.',
        listItem: true,
        type: CliMessageType.error,
      ),
    );
  }

  String get _gitIgnore => '''
# Files and directories created by pub.
.dart_tool/
.packages

# Conventional directory for build output.
build/
''';

  String get _readmeDotMd => '''
<img
    src="https://raw.githubusercontent.com/Hawmex/dawn/main/example/web/assets/banner.svg"
    width="100%"
/>

---

# $_appName

A Dawn App.

[ [Visit Dawn's official website →](https://dawn-dev.netlify.app) ]

## Serve Your App

```
webdev serve
```

[ [Read more →](https://dawn-dev.netlify.app/get-started) ]

''';

  String get _pubspecDotYaml => '''
name: $_appName
description: A Dawn app
publish_to: none
environment:
  sdk: ">=2.18.0 <3.0.0"
''';

  String get _analysisOptionsDotYaml => '''
include: package:dawn_lints/dawn_lints.yaml
''';

  String get _indexDotHtml => '''
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />

    <title>$_appName</title>

    <link rel="shortcut icon" href="/assets/icon.svg" type="image/x-icon" />

    <style>
      *,
      *::before,
      *::after {
        margin: 0px;
        padding: 0px;
        -webkit-tap-highlight-color: transparent;
        box-sizing: border-box;
      }
    </style>

    <script src="/main.dart.js" defer></script>
  </head>

  <body></body>
</html>
''';

  String get _mainDotDart => '''
import 'package:dawn/dawn.dart';

void main() => runApp(const App());

class App extends StatelessWidget {
  const App({super.key});

  @override
  Widget build(final BuildContext context) {
    return const Container(
      [
        Image(
          '/assets/logo.svg',
          style: Style({'width': '128px', 'height': '128px'}),
          animation: Animation(
            keyframes: [
              Keyframe(
                offset: 0,
                style: Style({'transform': 'translateY(0px)'}),
              ),
              Keyframe(
                offset: 1,
                style: Style({'transform': 'translateY(8px)'}),
              ),
            ],
            duration: Duration(seconds: 1),
            easing: Easing(0.2, 0, 0.4, 1),
            direction: AnimationDirection.alternate,
            iterations: double.infinity,
          ),
        ),
        Text(
          'Welcome to Dawn',
          style: Style({
            'font-size': '24px',
            'font-weight': 'bold',
            'color': '#00e690',
          }),
        ),
        Container([
          Text('To get started, edit '),
          Text(
            'web/main.dart',
            style: Style({
              'font-family': 'monospace',
              'background': '#eeeeee',
              'border-radius': '4px',
              'padding': '4px',
            }),
          ),
          Text(' and save to reload.'),
        ]),
      ],
      style: Style({
        'display': 'flex',
        'flex-flow': 'column',
        'justify-content': 'center',
        'text-align': 'center',
        'align-items': 'center',
        'gap': '16px',
        'padding': '16px',
        'width': '100%',
        'min-height': '100vh',
        'background': '#ffffff',
        'color': '#000000',
        'font-family': '"Jost", system-ui',
        'user-select': 'none',
      }),
    );
  }
}
''';

  String get _logoDotSvg => '''
<svg width="128" height="128" viewBox="0 0 128 128" fill="none" xmlns="http://www.w3.org/2000/svg">
    <g clip-path="url(#clip0_3_11)">
        <path d="M60.2426 12.7279C57.8995 10.3848 57.8995 6.58578 60.2426 4.24263V4.24263C62.5858 1.89949 66.3848 1.89949 68.7279 4.24263L123.882 59.397C126.225 61.7401 126.225 65.5391 123.882 67.8822L119.64 72.1249L60.2426 12.7279Z" fill="#7900EA" />
        <path d="M59.397 4.24264C61.7401 1.89949 65.5391 1.89949 67.8823 4.24264V4.24264C70.2254 6.58579 70.2254 10.3848 67.8823 12.7279L8.48528 72.1249L4.24264 67.8822C1.8995 65.5391 1.8995 61.7401 4.24264 59.397L59.397 4.24264Z" fill="#7900EA" />
        <rect x="58" y="25" width="12" height="46" rx="6" fill="#7900EA" />
        <rect x="86" y="101" width="12" height="12" rx="6" fill="#7900EA" />
        <rect x="58" y="74" width="12" height="24" rx="6" fill="#7900EA" />
        <rect x="30" y="101" width="12" height="24" rx="6" fill="#7900EA" />
        <rect x="30" y="53" width="12" height="45" rx="6" fill="#7900EA" />
        <rect x="86" y="53" width="12" height="45" rx="6" fill="#7900EA" />
    </g>
    <defs>
        <clipPath id="clip0_3_11">
            <rect width="128" height="128" fill="white" />
        </clipPath>
    </defs>
</svg>
''';

  String get _iconDotSvg => '''
<svg width="128" height="128" viewBox="0 0 128 128" fill="none" xmlns="http://www.w3.org/2000/svg">
    <rect width="128" height="128" rx="56" fill="white" />
    <path d="M61.1379 26.5366C59.3823 24.781 59.3823 21.9345 61.1379 20.1789V20.1789C62.8936 18.4232 65.7401 18.4232 67.4957 20.1789L107.757 60.4405C110.101 62.7837 110.101 66.5827 107.757 68.9258L105.642 71.0409L61.1379 26.5366Z" fill="#7900EA" />
    <path d="M60.5043 20.1789C62.2599 18.4232 65.1064 18.4232 66.8621 20.1789V20.1789C68.6177 21.9345 68.6177 24.781 66.8621 26.5366L22.3578 71.0409L20.2426 68.9258C17.8995 66.5827 17.8995 62.7837 20.2426 60.4405L60.5043 20.1789Z" fill="#7900EA" />
    <rect x="59.4576" y="35.7317" width="8.99123" height="34.4664" rx="4.49561" fill="#7900EA" />
    <rect x="80.4371" y="92.6762" width="8.99123" height="8.99123" rx="4.49561" fill="#7900EA" />
    <rect x="59.4576" y="72.4459" width="8.99123" height="17.9825" rx="4.49561" fill="#7900EA" />
    <rect x="38.4781" y="92.6762" width="8.99123" height="17.9825" rx="4.49561" fill="#7900EA" />
    <rect x="38.4781" y="56.7113" width="8.99123" height="33.7171" rx="4.49561" fill="#7900EA" />
    <rect x="80.4371" y="56.7113" width="8.99123" height="33.7171" rx="4.49561" fill="#7900EA" />
</svg>
''';

  String get _titleDotSvg => '''
<svg width="167" height="180" viewBox="0 0 167 180" fill="none" xmlns="http://www.w3.org/2000/svg">
    <g clip-path="url(#clip0_3_40)">
        <path d="M79.2426 12.7279C76.8995 10.3848 76.8995 6.58578 79.2426 4.24263V4.24263C81.5858 1.89949 85.3848 1.89949 87.7279 4.24263L142.882 59.397C145.225 61.7401 145.225 65.5391 142.882 67.8822L138.64 72.1249L79.2426 12.7279Z" fill="#7900EA" />
        <path d="M78.397 4.24264C80.7401 1.89949 84.5391 1.89949 86.8823 4.24264V4.24264C89.2254 6.58579 89.2254 10.3848 86.8823 12.7279L27.4853 72.1249L23.2426 67.8822C20.8995 65.5391 20.8995 61.7401 23.2426 59.397L78.397 4.24264Z" fill="#7900EA" />
        <rect x="77" y="25" width="12" height="46" rx="6" fill="#7900EA" />
        <rect x="105" y="101" width="12" height="12" rx="6" fill="#7900EA" />
        <rect x="77" y="74" width="12" height="24" rx="6" fill="#7900EA" />
        <rect x="49" y="101" width="12" height="24" rx="6" fill="#7900EA" />
        <rect x="49" y="53" width="12" height="45" rx="6" fill="#7900EA" />
        <rect x="105" y="53" width="12" height="45" rx="6" fill="#7900EA" />
    </g>
    <path d="M4.9235 131.2V176H16.0595V131.2H4.9235ZM20.9235 176C25.8302 176 30.0755 175.083 33.6595 173.248C37.2435 171.413 40.0168 168.832 41.9795 165.504C43.9422 162.133 44.9235 158.165 44.9235 153.6C44.9235 148.992 43.9422 145.024 41.9795 141.696C40.0168 138.368 37.2435 135.787 33.6595 133.952C30.0755 132.117 25.8302 131.2 20.9235 131.2H12.0915V140.8H20.5395C22.2462 140.8 23.8888 141.013 25.4675 141.44C27.0888 141.867 28.4968 142.571 29.6915 143.552C30.9288 144.533 31.9102 145.856 32.6355 147.52C33.3608 149.141 33.7235 151.168 33.7235 153.6C33.7235 156.032 33.3608 158.08 32.6355 159.744C31.9102 161.365 30.9288 162.667 29.6915 163.648C28.4968 164.629 27.0888 165.333 25.4675 165.76C23.8888 166.187 22.2462 166.4 20.5395 166.4H12.0915V176H20.9235ZM58.389 166.784C58.389 166.059 58.5597 165.461 58.901 164.992C59.2423 164.48 59.7543 164.096 60.437 163.84C61.1197 163.584 62.037 163.456 63.189 163.456C64.6823 163.456 66.0903 163.669 67.413 164.096C68.7783 164.48 69.973 165.056 70.997 165.824V161.536C70.485 160.981 69.717 160.448 68.693 159.936C67.669 159.424 66.4317 158.997 64.981 158.656C63.573 158.315 61.973 158.144 60.181 158.144C56.5117 158.144 53.6743 158.976 51.669 160.64C49.6637 162.261 48.661 164.48 48.661 167.296C48.661 169.301 49.1303 171.008 50.069 172.416C51.0503 173.824 52.3303 174.891 53.909 175.616C55.5303 176.299 57.2797 176.64 59.157 176.64C61.0343 176.64 62.7837 176.32 64.405 175.68C66.0263 174.997 67.349 174.016 68.373 172.736C69.397 171.413 69.909 169.813 69.909 167.936L68.885 164.096C68.885 165.461 68.5863 166.613 67.989 167.552C67.3917 168.491 66.6237 169.195 65.685 169.664C64.7463 170.133 63.7223 170.368 62.613 170.368C61.845 170.368 61.141 170.24 60.501 169.984C59.861 169.685 59.349 169.28 58.965 168.768C58.581 168.213 58.389 167.552 58.389 166.784ZM54.101 155.584C54.5277 155.328 55.2103 155.008 56.149 154.624C57.0877 154.197 58.1757 153.835 59.413 153.536C60.693 153.237 61.9943 153.088 63.317 153.088C64.213 153.088 65.0023 153.173 65.685 153.344C66.4103 153.515 67.0077 153.792 67.477 154.176C67.9463 154.56 68.2877 155.029 68.501 155.584C68.757 156.096 68.885 156.715 68.885 157.44V176H78.165V155.264C78.165 153.216 77.5677 151.488 76.373 150.08C75.1783 148.672 73.557 147.605 71.509 146.88C69.461 146.112 67.1357 145.728 64.533 145.728C61.7597 145.728 59.1997 146.091 56.853 146.816C54.5063 147.541 52.501 148.309 50.837 149.12L54.101 155.584ZM80.8645 146.56L96.0325 177.856L105.569 160.32L115.297 177.856L130.465 146.56H120.353L114.208 161.664L105.569 144.32L97.1205 161.664L90.9765 146.56H80.8645ZM152.912 158.08V176H162.96V157.056C162.96 153.515 162.128 150.763 160.464 148.8C158.8 146.837 156.133 145.856 152.464 145.856C150.288 145.856 148.453 146.304 146.96 147.2C145.509 148.096 144.357 149.355 143.504 150.976V146.56H133.776V176H143.504V158.08C143.504 156.928 143.717 155.947 144.144 155.136C144.571 154.325 145.168 153.707 145.936 153.28C146.704 152.853 147.6 152.64 148.624 152.64C150.16 152.64 151.248 153.109 151.888 154.048C152.571 154.987 152.912 156.331 152.912 158.08Z" fill="#00E690" />
    <defs>
        <clipPath id="clip0_3_40">
            <rect width="128" height="128" fill="white" transform="translate(19)" />
        </clipPath>
    </defs>
</svg>
''';

  String get _bannerDotSvg => '''
<svg width="720" height="360" viewBox="0 0 720 360" fill="none" xmlns="http://www.w3.org/2000/svg">
    <rect width="720" height="360" fill="white" />
    <path d="M340.568 142.4V176H348.92V142.4H340.568ZM352.568 176C356.248 176 359.432 175.312 362.12 173.936C364.808 172.56 366.888 170.624 368.36 168.128C369.832 165.6 370.568 162.624 370.568 159.2C370.568 155.744 369.832 152.768 368.36 150.272C366.888 147.776 364.808 145.84 362.12 144.464C359.432 143.088 356.248 142.4 352.568 142.4H345.944V149.6H352.28C353.56 149.6 354.792 149.76 355.976 150.08C357.192 150.4 358.248 150.928 359.144 151.664C360.072 152.4 360.808 153.392 361.352 154.64C361.896 155.856 362.168 157.376 362.168 159.2C362.168 161.024 361.896 162.56 361.352 163.808C360.808 165.024 360.072 166 359.144 166.736C358.248 167.472 357.192 168 355.976 168.32C354.792 168.64 353.56 168.8 352.28 168.8H345.944V176H352.568ZM380.667 169.088C380.667 168.544 380.795 168.096 381.051 167.744C381.307 167.36 381.691 167.072 382.203 166.88C382.715 166.688 383.403 166.592 384.267 166.592C385.387 166.592 386.443 166.752 387.435 167.072C388.459 167.36 389.355 167.792 390.123 168.368V165.152C389.739 164.736 389.163 164.336 388.395 163.952C387.627 163.568 386.699 163.248 385.611 162.992C384.555 162.736 383.355 162.608 382.011 162.608C379.259 162.608 377.131 163.232 375.627 164.48C374.123 165.696 373.371 167.36 373.371 169.472C373.371 170.976 373.723 172.256 374.427 173.312C375.163 174.368 376.123 175.168 377.307 175.712C378.523 176.224 379.835 176.48 381.243 176.48C382.651 176.48 383.963 176.24 385.179 175.76C386.395 175.248 387.387 174.512 388.155 173.552C388.923 172.56 389.307 171.36 389.307 169.952L388.539 167.072C388.539 168.096 388.315 168.96 387.867 169.664C387.419 170.368 386.843 170.896 386.139 171.248C385.435 171.6 384.667 171.776 383.835 171.776C383.259 171.776 382.731 171.68 382.251 171.488C381.771 171.264 381.387 170.96 381.099 170.576C380.811 170.16 380.667 169.664 380.667 169.088ZM377.451 160.688C377.771 160.496 378.283 160.256 378.987 159.968C379.691 159.648 380.507 159.376 381.435 159.152C382.395 158.928 383.371 158.816 384.363 158.816C385.035 158.816 385.627 158.88 386.139 159.008C386.683 159.136 387.131 159.344 387.483 159.632C387.835 159.92 388.091 160.272 388.251 160.688C388.443 161.072 388.539 161.536 388.539 162.08V176H395.499V160.448C395.499 158.912 395.051 157.616 394.155 156.56C393.259 155.504 392.043 154.704 390.507 154.16C388.971 153.584 387.227 153.296 385.275 153.296C383.195 153.296 381.275 153.568 379.515 154.112C377.755 154.656 376.251 155.232 375.003 155.84L377.451 160.688ZM397.523 153.92L408.899 177.392L416.051 164.24L423.347 177.392L434.723 153.92H427.139L422.531 165.248L416.051 152.24L409.715 165.248L405.107 153.92H397.523ZM451.559 162.56V176H459.095V161.792C459.095 159.136 458.471 157.072 457.223 155.6C455.975 154.128 453.975 153.392 451.223 153.392C449.591 153.392 448.215 153.728 447.095 154.4C446.007 155.072 445.143 156.016 444.503 157.232V153.92H437.207V176H444.503V162.56C444.503 161.696 444.663 160.96 444.983 160.352C445.303 159.744 445.751 159.28 446.327 158.96C446.903 158.64 447.575 158.48 448.343 158.48C449.495 158.48 450.311 158.832 450.791 159.536C451.303 160.24 451.559 161.248 451.559 162.56Z" fill="#00E690" />
    <path d="M287.844 133.517C286.645 132.317 286.645 130.372 287.844 129.172C289.044 127.973 290.989 127.973 292.189 129.172L318.357 155.341C320.7 157.684 320.7 161.483 318.357 163.826L318.255 163.928L287.844 133.517Z" fill="#7900EA" />
    <path d="M287.411 129.172C288.611 127.973 290.556 127.973 291.756 129.172C292.955 130.372 292.955 132.317 291.756 133.517L261.344 163.928L261.243 163.826C258.9 161.483 258.899 157.684 261.243 155.341L287.411 129.172Z" fill="#7900EA" />
    <rect x="286.696" y="139.8" width="6.144" height="23.552" rx="3.072" fill="#7900EA" />
    <rect x="301.032" y="178.712" width="6.144" height="6.144" rx="3.072" fill="#7900EA" />
    <rect x="286.696" y="164.888" width="6.144" height="12.288" rx="3.072" fill="#7900EA" />
    <rect x="272.36" y="178.712" width="6.144" height="12.288" rx="3.072" fill="#7900EA" />
    <rect x="272.36" y="154.136" width="6.144" height="23.04" rx="3.072" fill="#7900EA" />
    <rect x="301.032" y="154.136" width="6.144" height="23.04" rx="3.072" fill="#7900EA" />
    <path d="M237.572 228.76H246.452L246.188 225.664H237.86L237.572 228.76ZM241.964 221.368L244.34 226.816L244.196 227.848L246.212 232H250.82L241.964 214.144L233.132 232H237.716L239.804 227.68L239.612 226.792L241.964 221.368ZM259.776 215.2V232H263.952V215.2H259.776ZM265.776 232C267.616 232 269.208 231.656 270.552 230.968C271.896 230.28 272.936 229.312 273.672 228.064C274.408 226.8 274.776 225.312 274.776 223.6C274.776 221.872 274.408 220.384 273.672 219.136C272.936 217.888 271.896 216.92 270.552 216.232C269.208 215.544 267.616 215.2 265.776 215.2H262.464V218.8H265.632C266.272 218.8 266.888 218.88 267.48 219.04C268.088 219.2 268.616 219.464 269.064 219.832C269.528 220.2 269.896 220.696 270.168 221.32C270.44 221.928 270.576 222.688 270.576 223.6C270.576 224.512 270.44 225.28 270.168 225.904C269.896 226.512 269.528 227 269.064 227.368C268.616 227.736 268.088 228 267.48 228.16C266.888 228.32 266.272 228.4 265.632 228.4H262.464V232H265.776ZM279.826 228.544C279.826 228.272 279.89 228.048 280.018 227.872C280.146 227.68 280.338 227.536 280.594 227.44C280.85 227.344 281.194 227.296 281.626 227.296C282.186 227.296 282.714 227.376 283.21 227.536C283.722 227.68 284.17 227.896 284.554 228.184V226.576C284.362 226.368 284.074 226.168 283.69 225.976C283.306 225.784 282.842 225.624 282.298 225.496C281.77 225.368 281.17 225.304 280.498 225.304C279.122 225.304 278.058 225.616 277.306 226.24C276.554 226.848 276.178 227.68 276.178 228.736C276.178 229.488 276.354 230.128 276.706 230.656C277.074 231.184 277.554 231.584 278.146 231.856C278.754 232.112 279.41 232.24 280.114 232.24C280.818 232.24 281.474 232.12 282.082 231.88C282.69 231.624 283.186 231.256 283.57 230.776C283.954 230.28 284.146 229.68 284.146 228.976L283.762 227.536C283.762 228.048 283.65 228.48 283.426 228.832C283.202 229.184 282.914 229.448 282.562 229.624C282.21 229.8 281.826 229.888 281.41 229.888C281.122 229.888 280.858 229.84 280.618 229.744C280.378 229.632 280.186 229.48 280.042 229.288C279.898 229.08 279.826 228.832 279.826 228.544ZM278.218 224.344C278.378 224.248 278.634 224.128 278.986 223.984C279.338 223.824 279.746 223.688 280.21 223.576C280.69 223.464 281.178 223.408 281.674 223.408C282.01 223.408 282.306 223.44 282.562 223.504C282.834 223.568 283.058 223.672 283.234 223.816C283.41 223.96 283.538 224.136 283.618 224.344C283.714 224.536 283.762 224.768 283.762 225.04V232H287.242V224.224C287.242 223.456 287.018 222.808 286.57 222.28C286.122 221.752 285.514 221.352 284.746 221.08C283.978 220.792 283.106 220.648 282.13 220.648C281.09 220.648 280.13 220.784 279.25 221.056C278.37 221.328 277.618 221.616 276.994 221.92L278.218 224.344ZM293.531 220.96H289.955V232H293.531V220.96ZM296.723 224.704L298.283 221.608C298.091 221.32 297.811 221.096 297.443 220.936C297.075 220.776 296.691 220.696 296.291 220.696C295.683 220.696 295.099 220.896 294.539 221.296C293.979 221.696 293.523 222.24 293.171 222.928C292.835 223.616 292.667 224.4 292.667 225.28L293.531 226.552C293.531 226.024 293.603 225.576 293.747 225.208C293.891 224.84 294.099 224.56 294.371 224.368C294.643 224.176 294.955 224.08 295.307 224.08C295.627 224.08 295.891 224.136 296.099 224.248C296.323 224.36 296.531 224.512 296.723 224.704ZM298.526 220.96V223.96H305.918V220.96H298.526ZM300.47 217.12V232H303.95V217.12H300.47ZM313.074 220.96L318.762 232.696L322.338 226.12L325.986 232.696L331.674 220.96H327.882L325.578 226.624L322.338 220.12L319.17 226.624L316.866 220.96H313.074ZM338.18 232.24C339.588 232.24 340.78 231.968 341.756 231.424C342.748 230.88 343.532 230.064 344.108 228.976L340.868 228.184C340.596 228.664 340.228 229.024 339.764 229.264C339.316 229.504 338.772 229.624 338.132 229.624C337.572 229.624 337.1 229.504 336.716 229.264C336.332 229.008 336.044 228.64 335.852 228.16C335.66 227.68 335.564 227.104 335.564 226.432C335.58 225.696 335.676 225.088 335.852 224.608C336.044 224.112 336.324 223.744 336.692 223.504C337.06 223.264 337.516 223.144 338.06 223.144C338.492 223.144 338.868 223.24 339.188 223.432C339.508 223.624 339.756 223.896 339.932 224.248C340.108 224.6 340.196 225.016 340.196 225.496C340.196 225.608 340.164 225.752 340.1 225.928C340.052 226.088 339.988 226.224 339.908 226.336L340.772 225.232H333.956V227.248H344.036C344.068 227.136 344.084 227 344.084 226.84C344.084 226.664 344.084 226.488 344.084 226.312C344.084 225.128 343.852 224.12 343.388 223.288C342.924 222.456 342.252 221.824 341.372 221.392C340.492 220.944 339.412 220.72 338.132 220.72C336.852 220.72 335.74 220.952 334.796 221.416C333.868 221.88 333.156 222.544 332.66 223.408C332.164 224.272 331.916 225.296 331.916 226.48C331.916 227.648 332.172 228.664 332.684 229.528C333.196 230.392 333.916 231.064 334.844 231.544C335.788 232.008 336.9 232.24 338.18 232.24ZM349.922 213.28H346.298V232H349.922V213.28ZM358.946 226.48C358.946 225.232 358.69 224.184 358.178 223.336C357.682 222.488 357.034 221.84 356.234 221.392C355.45 220.944 354.602 220.72 353.69 220.72C352.778 220.72 351.962 220.952 351.242 221.416C350.522 221.88 349.954 222.544 349.538 223.408C349.122 224.256 348.914 225.28 348.914 226.48C348.914 227.664 349.122 228.688 349.538 229.552C349.954 230.416 350.522 231.08 351.242 231.544C351.962 232.008 352.778 232.24 353.69 232.24C354.602 232.24 355.45 232.016 356.234 231.568C357.034 231.12 357.682 230.472 358.178 229.624C358.69 228.76 358.946 227.712 358.946 226.48ZM355.13 226.48C355.13 227.056 355.01 227.56 354.77 227.992C354.53 228.408 354.21 228.736 353.81 228.976C353.41 229.2 352.97 229.312 352.49 229.312C352.074 229.312 351.666 229.2 351.266 228.976C350.882 228.752 350.562 228.432 350.306 228.016C350.05 227.584 349.922 227.072 349.922 226.48C349.922 225.888 350.05 225.384 350.306 224.968C350.562 224.536 350.882 224.208 351.266 223.984C351.666 223.76 352.074 223.648 352.49 223.648C352.97 223.648 353.41 223.768 353.81 224.008C354.21 224.232 354.53 224.56 354.77 224.992C355.01 225.408 355.13 225.904 355.13 226.48ZM367.346 220.96V223.96H374.882V220.96H367.346ZM374.474 216.76L375.938 214.096C375.778 213.936 375.562 213.768 375.29 213.592C375.018 213.4 374.69 213.24 374.306 213.112C373.922 212.968 373.45 212.896 372.89 212.896C371.978 212.896 371.21 213.056 370.586 213.376C369.978 213.696 369.514 214.152 369.194 214.744C368.89 215.336 368.738 216.048 368.738 216.88V232H372.218V217.792C372.218 217.408 372.266 217.096 372.362 216.856C372.458 216.616 372.586 216.44 372.746 216.328C372.922 216.216 373.13 216.16 373.37 216.16C373.578 216.16 373.77 216.208 373.946 216.304C374.138 216.4 374.314 216.552 374.474 216.76ZM380.156 220.96H376.58V232H380.156V220.96ZM383.348 224.704L384.908 221.608C384.716 221.32 384.436 221.096 384.068 220.936C383.7 220.776 383.316 220.696 382.916 220.696C382.308 220.696 381.724 220.896 381.164 221.296C380.604 221.696 380.148 222.24 379.796 222.928C379.46 223.616 379.292 224.4 379.292 225.28L380.156 226.552C380.156 226.024 380.228 225.576 380.372 225.208C380.516 224.84 380.724 224.56 380.996 224.368C381.268 224.176 381.58 224.08 381.932 224.08C382.252 224.08 382.516 224.136 382.724 224.248C382.948 224.36 383.156 224.512 383.348 224.704ZM389.255 228.544C389.255 228.272 389.319 228.048 389.447 227.872C389.575 227.68 389.767 227.536 390.023 227.44C390.279 227.344 390.623 227.296 391.055 227.296C391.615 227.296 392.143 227.376 392.639 227.536C393.151 227.68 393.599 227.896 393.983 228.184V226.576C393.791 226.368 393.503 226.168 393.119 225.976C392.735 225.784 392.271 225.624 391.727 225.496C391.199 225.368 390.599 225.304 389.927 225.304C388.551 225.304 387.487 225.616 386.735 226.24C385.983 226.848 385.607 227.68 385.607 228.736C385.607 229.488 385.783 230.128 386.135 230.656C386.503 231.184 386.983 231.584 387.575 231.856C388.183 232.112 388.839 232.24 389.543 232.24C390.247 232.24 390.903 232.12 391.511 231.88C392.119 231.624 392.615 231.256 392.999 230.776C393.383 230.28 393.575 229.68 393.575 228.976L393.191 227.536C393.191 228.048 393.079 228.48 392.855 228.832C392.631 229.184 392.343 229.448 391.991 229.624C391.639 229.8 391.255 229.888 390.839 229.888C390.551 229.888 390.287 229.84 390.047 229.744C389.807 229.632 389.615 229.48 389.471 229.288C389.327 229.08 389.255 228.832 389.255 228.544ZM387.647 224.344C387.807 224.248 388.063 224.128 388.415 223.984C388.767 223.824 389.175 223.688 389.639 223.576C390.119 223.464 390.607 223.408 391.103 223.408C391.439 223.408 391.735 223.44 391.991 223.504C392.263 223.568 392.487 223.672 392.663 223.816C392.839 223.96 392.967 224.136 393.047 224.344C393.143 224.536 393.191 224.768 393.191 225.04V232H396.671V224.224C396.671 223.456 396.447 222.808 395.999 222.28C395.551 221.752 394.943 221.352 394.175 221.08C393.407 220.792 392.535 220.648 391.559 220.648C390.519 220.648 389.559 220.784 388.679 221.056C387.799 221.328 387.047 221.616 386.423 221.92L387.647 224.344ZM416.233 224.896C416.233 224 416.121 223.24 415.897 222.616C415.673 221.992 415.305 221.52 414.793 221.2C414.297 220.864 413.617 220.696 412.753 220.696C412.001 220.696 411.329 220.856 410.737 221.176C410.145 221.496 409.649 221.944 409.249 222.52C409.025 221.928 408.657 221.48 408.145 221.176C407.633 220.856 406.945 220.696 406.081 220.696C405.329 220.696 404.681 220.864 404.137 221.2C403.609 221.536 403.193 222 402.889 222.592V220.96H399.385V232H402.889V225.28C402.889 224.832 402.953 224.456 403.081 224.152C403.225 223.848 403.417 223.624 403.657 223.48C403.913 223.32 404.201 223.24 404.521 223.24C405.017 223.24 405.369 223.416 405.577 223.768C405.801 224.104 405.913 224.608 405.913 225.28V232H409.561V225.28C409.561 224.832 409.625 224.456 409.753 224.152C409.897 223.848 410.089 223.624 410.329 223.48C410.585 223.32 410.873 223.24 411.193 223.24C411.689 223.24 412.041 223.416 412.249 223.768C412.473 224.104 412.585 224.608 412.585 225.28V232H416.233V224.896ZM424.688 232.24C426.096 232.24 427.288 231.968 428.264 231.424C429.256 230.88 430.04 230.064 430.616 228.976L427.376 228.184C427.104 228.664 426.736 229.024 426.272 229.264C425.824 229.504 425.28 229.624 424.64 229.624C424.08 229.624 423.608 229.504 423.224 229.264C422.84 229.008 422.552 228.64 422.36 228.16C422.168 227.68 422.072 227.104 422.072 226.432C422.088 225.696 422.184 225.088 422.36 224.608C422.552 224.112 422.832 223.744 423.2 223.504C423.568 223.264 424.024 223.144 424.568 223.144C425 223.144 425.376 223.24 425.696 223.432C426.016 223.624 426.264 223.896 426.44 224.248C426.616 224.6 426.704 225.016 426.704 225.496C426.704 225.608 426.672 225.752 426.608 225.928C426.56 226.088 426.496 226.224 426.416 226.336L427.28 225.232H420.464V227.248H430.544C430.576 227.136 430.592 227 430.592 226.84C430.592 226.664 430.592 226.488 430.592 226.312C430.592 225.128 430.36 224.12 429.896 223.288C429.432 222.456 428.76 221.824 427.88 221.392C427 220.944 425.92 220.72 424.64 220.72C423.36 220.72 422.248 220.952 421.304 221.416C420.376 221.88 419.664 222.544 419.168 223.408C418.672 224.272 418.424 225.296 418.424 226.48C418.424 227.648 418.68 228.664 419.192 229.528C419.704 230.392 420.424 231.064 421.352 231.544C422.296 232.008 423.408 232.24 424.688 232.24ZM430.894 220.96L436.583 232.696L440.159 226.12L443.807 232.696L449.495 220.96H445.703L443.398 226.624L440.159 220.12L436.991 226.624L434.687 220.96H430.894ZM449.737 226.48C449.737 227.632 450.009 228.64 450.553 229.504C451.097 230.368 451.849 231.04 452.809 231.52C453.769 232 454.849 232.24 456.049 232.24C457.249 232.24 458.321 232 459.265 231.52C460.225 231.04 460.977 230.368 461.521 229.504C462.081 228.64 462.361 227.632 462.361 226.48C462.361 225.312 462.081 224.296 461.521 223.432C460.977 222.568 460.225 221.896 459.265 221.416C458.321 220.936 457.249 220.696 456.049 220.696C454.849 220.696 453.769 220.936 452.809 221.416C451.849 221.896 451.097 222.568 450.553 223.432C450.009 224.296 449.737 225.312 449.737 226.48ZM453.457 226.48C453.457 225.904 453.577 225.408 453.817 224.992C454.057 224.576 454.369 224.256 454.753 224.032C455.137 223.808 455.569 223.696 456.049 223.696C456.513 223.696 456.937 223.808 457.321 224.032C457.721 224.256 458.033 224.576 458.257 224.992C458.497 225.408 458.617 225.904 458.617 226.48C458.617 227.056 458.497 227.552 458.257 227.968C458.033 228.368 457.721 228.68 457.321 228.904C456.937 229.128 456.513 229.24 456.049 229.24C455.569 229.24 455.137 229.128 454.753 228.904C454.369 228.68 454.057 228.368 453.817 227.968C453.577 227.552 453.457 227.056 453.457 226.48ZM468.117 220.96H464.541V232H468.117V220.96ZM471.309 224.704L472.869 221.608C472.677 221.32 472.397 221.096 472.029 220.936C471.661 220.776 471.277 220.696 470.877 220.696C470.269 220.696 469.685 220.896 469.125 221.296C468.565 221.696 468.109 222.24 467.757 222.928C467.421 223.616 467.253 224.4 467.253 225.28L468.117 226.552C468.117 226.024 468.189 225.576 468.333 225.208C468.477 224.84 468.685 224.56 468.957 224.368C469.229 224.176 469.541 224.08 469.893 224.08C470.213 224.08 470.477 224.136 470.685 224.248C470.909 224.36 471.117 224.512 471.309 224.704ZM474.384 213.28V232H477.864V213.28H474.384ZM481.872 220.96L477.144 225.52L482.424 232H486.696L481.488 225.52L486.216 220.96H481.872Z" fill="black" />
</svg>
''';
}
